@@ -1,22 +1,32 @@
-'''
-  Copyright European Organization for Nuclear Research (CERN)
-
-  Licensed under the Apache License, Version 2.0 (the "License");
-  You may not use this file except in compliance with the License.
-  You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
-
-  Authors:
-  - Vincent Garonne, <vincent.garonne@cern.ch>, 2012
-  - Martin Barisits, <martin.barisits@cern.ch>, 2013-2018
-  - Cedric Serfon, <cedric.serfon@cern.ch>, 2014-2015
-  - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
-  - Patrick Austin, <patrick.austin@stfc.ac.uk>, 2020
-  - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
-  - Ian Johnson, <ian.johnson@stfc.ac.uk>, 2021
-  - Radu Carpa <radu.carpa@cern.ch>, 2021
-
-  PY3K COMPATIBLE
-'''
+# -*- coding: utf-8 -*-
+# Copyright 2012-2021 CERN
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# Authors:
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2013
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2017
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2018
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2015
+# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
+# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
+# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
+# - Ian Johnson <ijjorama@gmail.com>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 from rucio.api.permission import has_permission
 from rucio.common.config import config_get_bool
@@ -283,20 +293,30 @@ def examine_replication_rule(rule_id, issuer, vo='def'):
     return result
 
 
-def move_replication_rule(rule_id, rse_expression, issuer, vo='def'):
+def move_replication_rule(rule_id, rse_expression, activity, source_replica_expression, issuer, vo='def'):
     """
     Move a replication rule to another RSE and, once done, delete the original one.
 
-    :param rule_id:             Rule to be moved.
-    :param rse_expression:      RSE expression of the new rule.
-    :param session:             The DB Session.
-    :param vo:                  The VO to act on.
-    :raises:                    RuleNotFound, RuleReplaceFailed
+    :param rule_id:                    Rule to be moved.
+    :param rse_expression:             RSE expression of the new rule.
+    :param activity:                   Activity of the new rule.
+    :param source_replica_expression:  Source-Replica-Expression of the new rule.
+    :param session:                    The DB Session.
+    :param vo:                         The VO to act on.
+    :raises:                           RuleNotFound, RuleReplaceFailed, InvalidRSEExpression, AccessDenied
     """
-    kwargs = {'rule_id': rule_id, 'rse_expression': rse_expression}
+    kwargs = {
+        'rule_id': rule_id,
+        'rse_expression': rse_expression,
+        'activity': activity,
+        'source_replica_expression': source_replica_expression
+    }
     if is_multi_vo() and not has_permission(issuer=issuer, vo=vo, action='access_rule_vo', kwargs=kwargs):
         raise AccessDenied('Account %s can not access rules at other VOs.' % (issuer))
     if not has_permission(issuer=issuer, vo=vo, action='move_rule', kwargs=kwargs):
         raise AccessDenied('Account %s can not move this replication rule.' % (issuer))
 
-    return rule.move_rule(rule_id=rule_id, rse_expression=rse_expression)
+    return rule.move_rule(rule_id=rule_id,
+                          rse_expression=rse_expression,
+                          activity=activity,
+                          source_replica_expression=source_replica_expression)
