@@ -57,7 +57,7 @@ if not multivo:
 
     if config.config_has_section('policy'):
         try:
-            POLICY = config.config_get('policy', 'package') + ".permission"
+            POLICY = config.config_get('policy', 'package', check_config_table=False) + ".permission"
         except (NoOptionError, NoSectionError):
             # fall back to old system for now
             POLICY = 'rucio.core.permission.' + FALLBACK_POLICY.lower()
@@ -85,7 +85,7 @@ def load_permission_for_vo(vo):
                 POLICY = GENERIC_FALLBACK
             POLICY = 'rucio.core.permission.' + POLICY.lower()
     else:
-        POLICY = 'rucio.common.permission.' + GENERIC_FALLBACK.lower()
+        POLICY = 'rucio.core.permission.' + GENERIC_FALLBACK.lower()
 
     try:
         module = importlib.import_module(POLICY)
