@@ -21,6 +21,7 @@
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
 # - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 """
 Abacus-Account is a daemon to update Account counters.
@@ -35,7 +36,7 @@ import traceback
 
 import rucio.db.sqla.util
 from rucio.common import exception
-from rucio.common.logging import setup_logging
+from rucio.common.logging import setup_logging, formatted_logger
 from rucio.common.utils import get_thread_with_periodic_running_function, daemon_sleep
 from rucio.core.account_counter import get_updated_account_counters, update_account_counter, fill_account_counter_history_table
 from rucio.core.heartbeat import live, die, sanity_check
@@ -48,10 +49,6 @@ def account_update(once=False, sleep_time=10):
     Main loop to check and update the Account Counters.
     """
 
-    logging.info('account_update: starting')
-
-    logging.info('account_update: started')
-
     # Make an initial heartbeat so that all abacus-account daemons have the correct worker number on the next try
     executable = 'abacus-account'
     hostname = socket.gethostname()
@@ -64,15 +61,18 @@ def account_update(once=False, sleep_time=10):
             # Heartbeat
             heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=current_thread)
 
+            prepend_str = 'account_update[%i/%i] : ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+            logger = formatted_logger(logging.log, prepend_str + '%s')
+
             # Select a bunch of rses for to update for this worker
             start = time.time()  # NOQA
             account_rse_ids = get_updated_account_counters(total_workers=heartbeat['nr_threads'],
                                                            worker_number=heartbeat['assign_thread'])
-            logging.debug('Index query time %f size=%d' % (time.time() - start, len(account_rse_ids)))
+            logger(logging.DEBUG, 'Index query time %f size=%d' % (time.time() - start, len(account_rse_ids)))
 
             # If the list is empty, sent the worker to sleep
             if not account_rse_ids and not once:
-                logging.info('account_update[%s/%s] did not get any work' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1))
+                logger(logging.INFO, 'did not get any work')
                 daemon_sleep(start_time=start, sleep_time=sleep_time, graceful_stop=graceful_stop)
             else:
                 for account_rse_id in account_rse_ids:
@@ -80,9 +80,9 @@ def account_update(once=False, sleep_time=10):
                         break
                     start_time = time.time()
                     update_account_counter(account=account_rse_id[0], rse_id=account_rse_id[1])
-                    logging.debug('account_update[%s/%s]: update of account-rse counter "%s-%s" took %f' % (heartbeat['assign_thread'], heartbeat['nr_threads'] - 1, account_rse_id[0], account_rse_id[1], time.time() - start_time))
+                    logger(logging.DEBUG, 'update of account-rse counter "%s-%s" took %f' % (account_rse_id[0], account_rse_id[1], time.time() - start_time))
         except Exception:
-            logging.error(traceback.format_exc())
+            logger(logging.ERROR, traceback.format_exc())
 
         if once:
             break
