@@ -40,8 +40,6 @@ Conveyor transfer submitter is a daemon to manage non-tape file transfers.
 from __future__ import division
 
 import logging
-import os
-import socket
 import threading
 import time
 from collections import defaultdict
@@ -50,12 +48,13 @@ from six.moves.configparser import NoOptionError
 
 import rucio.db.sqla.util
 from rucio.common import exception
-from rucio.common.config import config_get, config_get_bool
-from rucio.common.logging import formatted_logger, setup_logging
+from rucio.common.config import config_get, config_get_bool, config_get_int
+from rucio.common.logging import setup_logging
 from rucio.common.schema import get_schema_value
-from rucio.core import heartbeat, transfer as transfer_core
+from rucio.common.utils import PriorityQueue
+from rucio.core import transfer as transfer_core
 from rucio.core.monitor import MultiCounter, record_timer
-from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses
+from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, HeartbeatHandler
 from rucio.db.sqla.constants import RequestType
 from rucio.transfertool.fts3 import FTS3Transfertool
 from rucio.transfertool.globus import GlobusTransferTool
@@ -80,11 +79,16 @@ TRANSFERTOOL_CLASSES_BY_NAME = {
 def submitter(once=False, rses=None, partition_wait_time=10,
               bulk=100, group_bulk=1, group_policy='rule', source_strategy=None,
               activities=None, sleep_time=600, max_sources=4, archive_timeout_override=None,
-              filter_transfertool=FILTER_TRANSFERTOOL, transfertool=TRANSFER_TOOL, transfertype=TRANSFER_TYPE):
+              filter_transfertool=FILTER_TRANSFERTOOL, transfertool=TRANSFER_TOOL,
+              transfertype=TRANSFER_TYPE, ignore_availability=False):
     """
     Main loop to submit a new transfer primitive to a transfertool.
     """
 
+    try:
+        partition_hash_var = config_get('conveyor', 'partition_hash_var')
+    except NoOptionError:
+        partition_hash_var = None
     try:
         scheme = config_get('conveyor', 'scheme')
     except NoOptionError:
@@ -100,7 +104,7 @@ def submitter(once=False, rses=None, partition_wait_time=10,
         timeout = None
 
     try:
-        bring_online = config_get('conveyor', 'bring_online')
+        bring_online = config_get_int('conveyor', 'bring_online')
     except NoOptionError:
         bring_online = 43200
 
@@ -119,47 +123,48 @@ def submitter(once=False, rses=None, partition_wait_time=10,
     logging.debug("Maximum time in queue for different activities: %s", max_time_in_queue)
 
     activity_next_exe_time = defaultdict(time.time)
-    executable = "conveyor-submitter"
+    logger_prefix = executable = "conveyor-submitter"
     if activities:
         activities.sort()
         executable += '--activities ' + str(activities)
     if filter_transfertool:
         executable += ' --filter-transfertool ' + filter_transfertool
 
-    hostname = socket.getfqdn()
-    pid = os.getpid()
-    hb_thread = threading.current_thread()
-    heartbeat.sanity_check(executable=executable, hostname=hostname)
-    heart_beat = heartbeat.live(executable, hostname, pid, hb_thread)
-    prefix = 'conveyor-submitter[%i/%i] : ' % (heart_beat['assign_thread'], heart_beat['nr_threads'])
-    logger = formatted_logger(logging.log, prefix + '%s')
-    logger(logging.INFO, 'Submitter starting with timeout %s', timeout)
+    if activities is None:
+        activities = [None]
+    if rses:
+        rse_ids = [rse['id'] for rse in rses]
+    else:
+        rse_ids = None
 
-    if partition_wait_time:
-        time.sleep(partition_wait_time)  # To prevent running on the same partition if all the poller restart at the same time
-    heart_beat = heartbeat.live(executable, hostname, pid, hb_thread)
-    prefix = 'conveyor-submitter[%i/%i] : ' % (heart_beat['assign_thread'], heart_beat['nr_threads'])
-    logger = formatted_logger(logging.log, prefix + '%s')
-    logger(logging.INFO, 'Transfer submitter started')
+    with HeartbeatHandler(executable=executable, logger_prefix=logger_prefix) as heartbeat_handler:
+        logger = heartbeat_handler.logger
+        logger(logging.INFO, 'Submitter starting with timeout %s', timeout)
 
-    while not graceful_stop.is_set():
-        if activities is None:
-            activities = [None]
-        if rses:
-            rse_ids = [rse['id'] for rse in rses]
-        else:
-            rse_ids = None
+        if partition_wait_time:
+            graceful_stop.wait(partition_wait_time)
+
+        activity_next_exe_time = PriorityQueue()
         for activity in activities:
+            activity_next_exe_time[activity] = time.time()
+
+        while not graceful_stop.is_set() and activity_next_exe_time:
             try:
-                if activity_next_exe_time[activity] > time.time():
-                    graceful_stop.wait(1)
-                    continue
+                time_to_sleep = 0
+                if once:
+                    activity = activity_next_exe_time.pop()
+                else:
+                    activity = activity_next_exe_time.top()
+                    time_to_sleep = activity_next_exe_time[activity] - time.time()
+                    activity_next_exe_time[activity] = time.time() + 1
+                if time_to_sleep > 0:
+                    logger(logging.DEBUG, 'Switching to activity %s and sleeping %s seconds', activity, time_to_sleep)
+                    graceful_stop.wait(time_to_sleep)
+                else:
+                    logger(logging.DEBUG, 'Switching to activity %s', activity)
 
-                heart_beat = heartbeat.live(executable, hostname, pid, hb_thread, older_than=3600)
-                prefix = 'conveyor-submitter[%i/%i] : ' % (heart_beat['assign_thread'], heart_beat['nr_threads'])
-                logger = formatted_logger(logging.log, prefix + '%s')
+                heart_beat, logger = heartbeat_handler.live(older_than=3600)
 
-                logger(logging.INFO, 'Starting to get transfer transfers for %s', activity)
                 start_time = time.time()
 
                 transfertool_kwargs = {
@@ -180,6 +185,7 @@ def submitter(once=False, rses=None, partition_wait_time=10,
                 transfers = transfer_core.next_transfers_to_submit(
                     total_workers=heart_beat['nr_threads'],
                     worker_number=heart_beat['assign_thread'],
+                    partition_hash_var=partition_hash_var,
                     failover_schemes=failover_scheme,
                     limit=bulk,
                     activity=activity,
@@ -189,6 +195,7 @@ def submitter(once=False, rses=None, partition_wait_time=10,
                     transfertools_by_name={transfertool: TRANSFERTOOL_CLASSES_BY_NAME[transfertool]},
                     older_than=None,
                     request_type=RequestType.TRANSFER,
+                    ignore_availability=ignore_availability,
                     logger=logger,
                 )
                 total_transfers = len(list(hop for paths in transfers.values() for path in paths for hop in path))
@@ -196,38 +203,31 @@ def submitter(once=False, rses=None, partition_wait_time=10,
                 record_timer('daemons.conveyor.transfer_submitter.get_transfers.per_transfer', (time.time() - start_time) * 1000 / (total_transfers or 1))
                 GET_TRANSFERS_COUNTER.inc(total_transfers)
                 record_timer('daemons.conveyor.transfer_submitter.get_transfers.transfers', total_transfers)
-                logger(logging.INFO, 'Got %s transfers for %s in %s seconds', total_transfers, activity, time.time() - start_time)
+                logger(logging.INFO, '%sGot %s transfers for %s in %s seconds',
+                       'Slept %s seconds, then ' % time_to_sleep if time_to_sleep > 0 else '',
+                       total_transfers, activity, time.time() - start_time)
 
-                grouped_jobs = {}
                 for builder, transfer_paths in transfers.items():
                     transfertool_obj = builder.make_transfertool(logger=logger, **transfertool_kwargs.get(builder.transfertool_class, {}))
                     start_time = time.time()
-                    logger(logging.INFO, 'Starting to group transfers for %s (%s)', activity, transfertool_obj)
+                    logger(logging.DEBUG, 'Starting to group transfers for %s (%s)', activity, transfertool_obj)
                     grouped_jobs = transfertool_obj.group_into_submit_jobs(transfer_paths)
                     record_timer('daemons.conveyor.transfer_submitter.bulk_group_transfer', (time.time() - start_time) * 1000 / (len(transfer_paths) or 1))
 
-                    logger(logging.INFO, 'Starting to submit transfers for %s (%s)', activity, transfertool_obj)
+                    logger(logging.DEBUG, 'Starting to submit transfers for %s (%s)', activity, transfertool_obj)
                     for job in grouped_jobs:
-                        logger(logging.DEBUG, 'submitjob: %s' % job)
+                        logger(logging.DEBUG, 'submitjob: transfers=%s, job_params=%s' % ([str(t) for t in job['transfers']], job['job_params']))
                         submit_transfer(transfertool_obj=transfertool_obj, transfers=job['transfers'], job_params=job['job_params'], submitter='transfer_submitter',
                                         timeout=timeout, logger=logger)
 
-                if total_transfers < group_bulk:
-                    logger(logging.INFO, 'Only %s transfers for %s which is less than group bulk %s, sleep %s seconds', total_transfers, activity, group_bulk, sleep_time)
-                    if activity_next_exe_time[activity] < time.time():
-                        activity_next_exe_time[activity] = time.time() + sleep_time
+                if not once and total_transfers < group_bulk:
+                    logger(logging.DEBUG, 'Only %s transfers for %s which is less than group bulk %s, sleep %s seconds', total_transfers, activity, group_bulk, sleep_time)
+                    activity_next_exe_time[activity] = time.time() + sleep_time
+
             except Exception:
                 logger(logging.CRITICAL, 'Exception', exc_info=True)
-
-        if once:
-            break
-
-    logger(logging.INFO, 'Graceful stop requested')
-
-    heartbeat.die(executable, hostname, pid, hb_thread)
-
-    logger(logging.INFO, 'Graceful stop done')
-    return
+                if once:
+                    raise
 
 
 def stop(signum=None, frame=None):
@@ -239,7 +239,7 @@ def stop(signum=None, frame=None):
 
 def run(once=False, group_bulk=1, group_policy='rule', mock=False,
         rses=None, include_rses=None, exclude_rses=None, vos=None, bulk=100, source_strategy=None,
-        activities=None, exclude_activities=None, sleep_time=600, max_sources=4,
+        activities=None, exclude_activities=None, ignore_availability=False, sleep_time=600, max_sources=4,
         archive_timeout_override=None, total_threads=1):
     """
     Starts up the conveyer threads.
@@ -285,6 +285,7 @@ def run(once=False, group_bulk=1, group_policy='rule', mock=False,
                                                           'group_bulk': group_bulk,
                                                           'group_policy': group_policy,
                                                           'activities': activities,
+                                                          'ignore_availability': ignore_availability,
                                                           'sleep_time': sleep_time,
                                                           'max_sources': max_sources,
                                                           'source_strategy': source_strategy,
