@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2021 CERN
+# Copyright 2021-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,8 +17,10 @@
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2021
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2021
 # - Eric Vaandering <ewv@fnal.gov>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
+# - Ilija Vukotic <ivukotic@cern.ch>, 2021
 # - Martin Barisits <martin.barisits@cern.ch>, 2021
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 from datetime import datetime
 from itertools import chain
@@ -44,6 +46,23 @@ from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask,
     request_auth_env, response_headers, generate_http_error_flask, ErrorHandlingMethodView, json_parameters, param_get
 
 
+def _sorted_with_priorities(replicas, sorted_pfns, limit=None):
+    """
+    Pick up to "limit" replicas from "replicas" in the order given by sorted_pfns.
+    Sets the corresponding priority in returned replicas.
+
+    :param replicas: Dictionary {pfn: replica_definition}.
+    :param sorted_pfns: Sorted list of pfns.
+    :param limit: only return this many replicas
+    :yields: index and corresponding pfn
+    """
+    for idx, pfn in enumerate(sorted_pfns, start=1):
+        if limit is None or idx <= limit:
+            replica = replicas[pfn]
+            replica['priority'] = idx
+            yield pfn, replica
+
+
 class Replicas(ErrorHandlingMethodView):
 
     @check_accept_header_wrapper_flask(['application/x-json-stream', 'application/metalink4+xml'])
@@ -116,6 +135,7 @@ class Replicas(ErrorHandlingMethodView):
                     replicas = sort_replicas(dictreplica, client_location, selection=select)
 
                     if not metalink:
+                        rfile['pfns'] = dict(_sorted_with_priorities(rfile['pfns'], replicas))
                         yield dumps(rfile) + '\n'
                     else:
                         yield ' <file name="' + rfile['name'] + '">\n'
@@ -364,17 +384,12 @@ class ListReplicas(ErrorHandlingMethodView):
                             lanreplicas[pfn] = replica_tuple
                         else:
                             wanreplicas[pfn] = replica_tuple
-                    unsorted_replicas = rfile['pfns']
-                    sorted_replicas = rfile['pfns'] = {}
-                    # Lan replicas sorted by priority; followed by wan replicas sorted by selection criteria
-                    for idx, pfn in enumerate(chain(sorted(lanreplicas.keys(), key=lambda pfn: lanreplicas[pfn][1]),
-                                                    sort_replicas(wanreplicas, client_location, selection=select)),
-                                              start=1):
-                        replica = unsorted_replicas[pfn]
-                        sorted_replicas[pfn] = replica
-                        replica['priority'] = idx
-                        if limit and limit == idx:
-                            break
+
+                    rfile['pfns'] = dict(_sorted_with_priorities(replicas=rfile['pfns'],
+                                                                 # Lan replicas sorted by priority; followed by wan replicas sorted by selection criteria
+                                                                 sorted_pfns=chain(sorted(lanreplicas.keys(), key=lambda pfn: lanreplicas[pfn][1]),
+                                                                                   sort_replicas(wanreplicas, client_location, selection=select)),
+                                                                 limit=limit))
 
                     if not metalink:
                         yield dumps(rfile, cls=APIEncoder) + '\n'
